﻿package com.mm.android.avplaysdk;

import com.mm.android.audiorecord.IVoiceIntercomListener;
import com.mm.android.audiorecord.VoiceIntercom;
import com.mm.android.avplaysdk.param.AVFrameInfo;

public class AVPlaySDK
{
	////////////////////////////////////////////////////////////////////////////////
	// 常量

	/**没有错误*/
	public static final int AVPLAY_OK = 0;
	
	/**未知错误*/
	public static final int AVPLAY_ERROR = -1;
	
	/**系统错误*/
	public static final int AVPLAY_SYSTEM_ERROR = -2;
	
	/**句柄无效*/
	public static final int AVPLAY_INVALID_HANDLE = -3; 
	
	/**参数不合法*/
	public static final int AVPLAY_ILLEGAL_PARAM = -4; 
	
	/**不支持此分辨率*/
	public static final int AVPLAY_RESOLUTION_UNSUPPORT = -5; 
	
	/**不支持此码流*/
	public static final int AVPLAY_STREAM_UNSUPPORT = -6;
	
	/**文件错误*/
	public static final int AVPLAY_FILE_ERROR = -7;
	
	/**没有SD卡*/
	public static final int AVPLAY_NO_SDCARD = -8;
	
	/**抓图失败*/
	public static final int AVPLAY_SNAP_FAILED	= -9;
	
	/**解码器初始化失败*/
	public static final int AVPLAY_DECODER_INIT_FAILED = -10;
	
	/**回放初始化失败*/
	public static final int AVPLAY_PLAYBACK_INIT_FAILED = -11;
	
	/**实时播放*/
	public static final int AV_STREAM_MODE_REAL_TIME = 0;
	
	/**回放，包括网络回放和自己读文件送数据的回放*/
	public static final int AV_STREAM_MODE_PLAY_BACK = 1;
	
	/**SDK版本*/
	private static final String m_strSDKVersion = "1.00.0";
	
	/**播放时允许的最大延时时间，单位为ms*/
	private static final int MAX_DELAY_TIME = 2000;
	
	////////////////////////////////////////////////////////////////////////////////
	// 结构
	
	/**
	 * 采样率
	 */
	public enum SAMPLE_PERSEC
	{
		/**采样率为8K*/
		SAMPLE_PERSEC_8000,
		
		/**采样率为16K*/
		SAMPLE_PERSEC_16000,
		
		/**采样率为48K*/
		SAMPLE_PERSEC_48000,
	}
	
	/**
	 * 采样深度
	 */
	public enum BIT_PERSAMPLE
	{
		/**采样深度为8位*/
		BIT_PERSAMPLE_8,
		
		/**采样深度为16位*/
		BIT_PERSAMPLE_16
	}
	
	/**
	 * 通道数
	 */
	public enum CHANNEL
	{
		/**单通道*/
		CHANNEL_MONO,
		/**双通道*/
		CHANNEL_STEREO
	}
	
	/**
	 * 音频编码类型
	 */
	public enum AUDIO_ENC_TYPE
	{
		/**音频的编码类型为PCM8*/
		AET_PCM8,
		
		/**音频的编码类型为PCM16*/
		AET_PCM16,
		
		/**音频的编码类型为G711A*/
		AET_G711A,
		
		/**音频的编码类型为G711U*/
		AET_G711U
	}
	
	/**
	 * 文件头，目前暂不使用
	 */
	public class AV_FILE_HEADER
	{
		/**包含文件头的buffer*/
		public byte[] fileHeadBuf = null;
		
		/**文件头在fileHeadBuf中的起始位置*/
		public int nOffset = 0;
		
		/**文件头的长度*/
		public int nHeadLen = 0;
	}
	
	////////////////////////////////////////////////////////////////////////////////
	// 接口
	
	/**
	 * SDK初始化，必须在调用其他接口之前调用
	 * @param strAppPath 应用程序的完整包名，使得播放SDK能正确地初始化
	 * @return 成功返回true，失败返回false
	 */
	public static final boolean startup(String strAppPath)
	{
		String playBackDir = strAppPath.substring(strAppPath.lastIndexOf("."));
		int nInitRetValue = AVPlayer.startup(strAppPath,playBackDir);
		AVErrorCenter.put(nInitRetValue);
		if (nInitRetValue != AVPLAY_OK)
		{
			return false;
		}

		return true;
	}
	
	/**
	 * SDK退出清理，在不再使用该播放SDK时调用
	 * @return 成功返回true，失败返回false
	 */
	public static final boolean cleanup()
	{
		return AVPlayer.cleanup();
	}

	/**
	 * 获取SDK版本信息
	 * @return 播放SDK的版本
	 */
	public static final String getSDKVersion()
	{
		return m_strSDKVersion;
	}

	/**
	 * 获取错误码
	 * @return 见错误码表
	 */
	public static final int getLastError()
	{
		return AVErrorCenter.get();
	}
	
	/**
	 * 打开流播放，在调用其他播放控制接口之前调用
	 * @param fileHead 文件头，目前传null
	 * @param nDelayTime 播放过程中指定的最大延时时间，单位为ms
	 * @param nType STREAM_MODE_REAL_TIME：实时，STREAM_MODE_PLAY_BACK：回放
	 * @param bPureAudio 标识是否为纯音频
	 * @param inputDataListener 实时监视时可为null，回放时不能为null
	 * @return 成功返回播放句柄，失败返回null
	 */
	public static final IPlayHandle openStream(AV_FILE_HEADER fileHead, int nDelayTime, 
											   int nType, boolean bPureAudio,
											   IInputDataListener inputDataListener)
	{
		if ((AV_STREAM_MODE_PLAY_BACK == nType) && (null == inputDataListener))
		{
			AVErrorCenter.put(AVPLAY_ILLEGAL_PARAM);
			return null;
		}
		
		AVPlayer avPlayer = getAVPlayer(fileHead, nType, inputDataListener);
		if (null == avPlayer)
		{
			AVErrorCenter.put(AVPLAY_SYSTEM_ERROR);
			return null;
		}
		
		if (!avPlayer.open(nDelayTime, bPureAudio))
		{
			AVErrorCenter.put(AVPLAY_SYSTEM_ERROR);
			avPlayer = null;
		}
		
		return avPlayer;
	}
	
	/**
	 * 关闭流播放
	 * @param hPlayHandle 播放句柄，由openStream返回
	 */
	public static final void closeStream(IPlayHandle hPlayHandle)
	{
		if (null != hPlayHandle)
		{
			AVPlayer avPlayer = (AVPlayer)hPlayHandle;
			avPlayer.close();
		}
	}
	
	/**
	 * 开始播放
	 * @param hPlayHandle 播放句柄，由openStream或openFile返回
	 * @param playView 播放窗口
	 * @param playListener 播放时间监听器
	 * @return 成功返回true，失败返回false
	 */
	public static final boolean play(IPlayHandle hPlayHandle, IPlayView playView, IPlayListener playListener)
	{
		if (null == hPlayHandle)
		{
			AVErrorCenter.put(AVPLAY_INVALID_HANDLE);
			return false;
		}
		
		AVPlayer avPlayer = (AVPlayer)hPlayHandle;
		
		return avPlayer.play(playView, playListener);
	}
	
	/**
	 * 停止播放
	 * @param hPlayHandle 播放句柄，由openStream或openFile返回
	 * @return 成功返回true，失败返回false
	 */
	public static final boolean stop(IPlayHandle hPlayHandle)
	{
		if (null == hPlayHandle)
		{
			AVErrorCenter.put(AVPLAY_INVALID_HANDLE);
			return false;
		}
		
		AVPlayer avPlayer = (AVPlayer)hPlayHandle;
		
		return avPlayer.stop();
	}
	
	/**
	 * 重置播放器
	 * @param hPlayHandle
	 * @return
	 */
	public static final boolean resetPlayer(IPlayHandle hPlayHandle) {
		if (null == hPlayHandle)
		{
			AVErrorCenter.put(AVPLAY_INVALID_HANDLE);
			return false;
		}
		
		AVPlayer avPlayer = (AVPlayer)hPlayHandle;
		
		return avPlayer.resetPlayer();
	}
	
	/**
	 * 设置是否按帧播放
	 * @param hPlayHandle
	 * @param byFrame
	 */
	public static final void setPlayOneFrameMode(IPlayHandle hPlayHandle,boolean byFrame)
	{
		if (null == hPlayHandle)
		{
			AVErrorCenter.put(AVPLAY_INVALID_HANDLE);
			return;
		}
		AVPlayer avPlayer = (AVPlayer)hPlayHandle;
		avPlayer.setPlayOneFrameMode(byFrame);
	}
	
	/**
	 * 播放一帧图像
	 * @param hPlayHandle
	 */
	public static final void playOneFrame(IPlayHandle hPlayHandle)
	{
		if (null == hPlayHandle)
		{
			AVErrorCenter.put(AVPLAY_INVALID_HANDLE);
			return;
		}
		AVPlayer avPlayer = (AVPlayer)hPlayHandle;
		avPlayer.playOneFrame();
	}
	
	/**
	 * 输入数据流，在回放过程中，如果返回false，那么应该停止向SDK送入码流数据，等到SDK通过IInputDataListener发来通知时再继续送入数据
	 * @param hPlayHandle 播放句柄，由openStream返回
	 * @param streamData 网络中接收或者从文件中读出的码流数据
	 * @param nOffset 码流数据在streamData中的偏移量
	 * @param nDataLen 码流数据长度
	 * @return 成功返回true，失败返回false
	 */
	public static final boolean inputData(IPlayHandle hPlayHandle, byte[] streamData, int nOffset, int nDataLen)
	{
		return inputData(hPlayHandle, streamData, nOffset, nDataLen, true);
	}
	public static final boolean inputData(IPlayHandle hPlayHandle, byte[] streamData, int nOffset, int nDataLen, boolean bPlay)
	{
		if ((null == hPlayHandle) || (null == streamData) || (0 == nDataLen))
		{
			AVErrorCenter.put(AVPLAY_ILLEGAL_PARAM);
			return false;
		}
		
		AVPlayer avPlayer = (AVPlayer)hPlayHandle;
		
		return avPlayer.inputData(streamData, nOffset, nDataLen, bPlay);
	}
	
	/**
	 * 输入帧数据流，在回放过程中，如果返回false，那么应该停止向SDK送入码流数据，等到SDK通过IInputDataListener发来通知时再继续送入数据
	 * @param hPlayHandle 播放句柄，由openStream返回
	 * @param hFrameInfo 网络中接收或者从文件中读出的帧数据
	 * @return 成功返回true，失败返回false
	 */
	public static final boolean inputFrame(IPlayHandle hPlayHandle, AVFrameInfo hFrameInfo)
	{
		if ((null == hPlayHandle) || (null == hFrameInfo))
		{
			AVErrorCenter.put(AVPLAY_ILLEGAL_PARAM);
			return false;
		}
		
		AVPlayer avPlayer = (AVPlayer)hPlayHandle;
		
		return avPlayer.inputFrame(hFrameInfo);
	}
	
	/**
	 * 打开文件播放，在调用其他播放控制接口之前调用
	 * @param strFileName 打开文件所在绝对路径
	 * @param bPureAudio 所打开的文件是否是纯音频文件
	 * @return 成功返回播放句柄，失败返回null
	 */
	public static final IPlayHandle openFile(String strFileName, boolean bPureAudio)
	{
		if (null == strFileName)
		{
			AVErrorCenter.put(AVPLAY_ILLEGAL_PARAM);
			return null;
		}
		
		AVPlayer avPlayer = new AVFilePlayer(strFileName);
		if (!avPlayer.open(MAX_DELAY_TIME, bPureAudio))
		{
			AVErrorCenter.put(AVPLAY_SYSTEM_ERROR);
			avPlayer = null;
			return null;
		}
		
		return avPlayer; 
	}
	
	/**
	 * 关闭文件播放
	 * @param hPlayHandle 播放句柄，由openFile返回
	 */
	public static final void closeFile(IPlayHandle hPlayHandle)
	{
		if (null != hPlayHandle)
		{
			AVPlayer avPlayer = (AVPlayer)hPlayHandle;
			avPlayer.close();
		}
	}
	
	/**
	 * 回放时拖动定位功能
	 * @param hPlayHandle 播放句柄，由openStream或openFile返回，实时监视时不能定位
	 * @param nPlayTime 定位的绝对时间，单位为秒
	 * @return 成功返回true，失败返回false
	 */
	public static final boolean setPlayPos(IPlayHandle hPlayHandle, int nPlayTime)
	{
		if ((null == hPlayHandle) || (nPlayTime < 0))
		{
			AVErrorCenter.put(AVPLAY_ILLEGAL_PARAM);
			return false;
		}
		
		AVPlayer avPlayer = (AVPlayer)hPlayHandle;
		
		return avPlayer.setPlayPos(nPlayTime);
	}
	
	/**
	 * 暂停与继续
	 * @param hPlayHandle 播放句柄，由openStream或openFile返回，实时监视时不能暂停
	 * @param bPause
	 * @return 成功返回true，失败返回false
	 */
	public static final boolean pause(IPlayHandle hPlayHandle, boolean bPause)
	{
		if (null == hPlayHandle)
		{
			AVErrorCenter.put(AVPLAY_INVALID_HANDLE);
			return false;
		}
		
		AVPlayer avPlayer = (AVPlayer)hPlayHandle;
		
		return avPlayer.pause(bPause);
	}
	
	/**
	 * 调整播放速度，实时监视时不能调整播放速度
	 * @param hPlayHandle 播放句柄，由openStream或openFile返回
	 * @param nMultiSpeed -1：慢放1倍，0：正常速度，1：快放1倍
	 * @return 成功返回true，失败返回false
	 */
	public static final boolean setPlaySpeed(IPlayHandle hPlayHandle, int nMultiSpeed)
	{
		if ((null == hPlayHandle) || !( (-1 == nMultiSpeed) || 
			(0 == nMultiSpeed) || (1 == nMultiSpeed)))
		{
			AVErrorCenter.put(AVPLAY_ILLEGAL_PARAM);
			return false;
		}
		
		AVPlayer avPlayer = (AVPlayer)hPlayHandle;
		
		return avPlayer.setPlaySpeed(nMultiSpeed);
	}
	
	/**
	 * 获取当前播放速度
	 * @param hPlayHandle 播放句柄，由openStream或openFile返回
	 * @return 成功返回播放速度，失败返回 -1
	 */
	public static final int getPlaySpeed(IPlayHandle hPlayHandle)
	{
		if(null == hPlayHandle)
		{
			AVErrorCenter.put(AVPLAY_ILLEGAL_PARAM);
			return -1;
		}
		
		AVPlayer avPlayer = (AVPlayer)hPlayHandle;
		return avPlayer.getPlaySpeed();
	}
	
	
	/**
	 * 开始录像
	 * @param hPlayHandle	播放句柄
	 * @param strFilePath	录像文件
	 * @return
	 */
	public static final boolean startRecord(IPlayHandle hPlayHandle, String strFilePath)
	{
		if ((null == hPlayHandle) || (null == strFilePath))
		{
			AVErrorCenter.put(AVPLAY_ILLEGAL_PARAM);
			return false;
		}
		AVPlayer avPlayer = (AVPlayer)hPlayHandle;
		
		return avPlayer.startRecord(strFilePath);
	}
	
	/**
	 * 停止录像
	 * @param hPlayHandle	播放句柄
	 */
	public static final void stopRecord(IPlayHandle hPlayHandle)
	{
		if(hPlayHandle == null)
		{
			return;
		}
		AVPlayer avPlayer = (AVPlayer)hPlayHandle;
		
		avPlayer.stopRecord();
	}
	
	/**
	 * 抓图，保存格式为jpg
	 * @param hPlayHandle 播放句柄，由openStream或openFile返回
	 * @param strFileName 保存图片的绝对路径
	 * @param nQuality 图像质量
	 * @return 成功返回true，失败返回false
	 */
	public static final boolean snapPicture(IPlayHandle hPlayHandle, String strFileName, int nQuality)
	{
		if ((null == hPlayHandle) || (null == strFileName) || (nQuality < 0))
		{
			AVErrorCenter.put(AVPLAY_ILLEGAL_PARAM);
			return false;
		}
		
		AVPlayer avPlayer = (AVPlayer)hPlayHandle;
		
		return avPlayer.snapPicture(strFileName, nQuality);
	}
	
	/**
	 * 打开音频
	 * @param hPlayHandle 播放句柄，由openStream或openFile返回
	 * @return 成功返回true，失败返回false
	 */
	public static final boolean openAudio(IPlayHandle hPlayHandle)
	{
		if (null == hPlayHandle)
		{
			AVErrorCenter.put(AVPLAY_INVALID_HANDLE);
			return false;
		}
		
		AVPlayer avPlayer = (AVPlayer)hPlayHandle;
		
		return avPlayer.openAudio();
	}
	
	/**
	 * 关闭音频
	 * @param hPlayHandle 播放句柄，由openStream或openFile返回
	 * @return 成功返回true，失败返回false
	 */
	public static final boolean closeAudio(IPlayHandle hPlayHandle)
	{
		if (null == hPlayHandle)
		{
			AVErrorCenter.put(AVPLAY_INVALID_HANDLE);
			return false;
		}
		
		AVPlayer avPlayer = (AVPlayer)hPlayHandle;
		
		return avPlayer.closeAudio();
	}
	
	/**
	 * 获取音量0~100
	 * @param hPlayHandle 播放句柄，由openStream或openFile返回
	 * @return 音量值
	 */
	public static final short getAudioVolume(IPlayHandle hPlayHandle)
	{
		if (null == hPlayHandle)
		{
			AVErrorCenter.put(AVPLAY_INVALID_HANDLE);
			return -1;
		}
		
		AVPlayer avPlayer = (AVPlayer)hPlayHandle;
		
		return avPlayer.getAudioVolume();
	}
	
	/**
	 * 设置音量
	 * @param hPlayHandle 播放句柄，由openStream或openFile返回
	 * @param nVolume 设置的音量值
	 * @return 成功返回true，失败返回false
	 */
	public static final boolean setAudioVolume(IPlayHandle hPlayHandle, short nVolume)
	{
		if (null == hPlayHandle)
		{
			AVErrorCenter.put(AVPLAY_INVALID_HANDLE);
			return false;
		}
		
		AVPlayer avPlayer = (AVPlayer)hPlayHandle;
		
		return avPlayer.setAudioVolume(nVolume);
	}
	
	/**
	 * 设置是否须要显示，主要是在全屏一个画面时调用，这样可以减少资源的占用
	 * @param hPlayHandle 播放句柄，由openStream或openFile返回
	 * @param bRender true显示，false不显示
	 */
	public static final void enableRender(IPlayHandle hPlayHandle, boolean bRender)
	{
		if (null != hPlayHandle)
		{
			AVPlayer avPlayer = (AVPlayer)hPlayHandle;
			avPlayer.enableRender(bRender);
		}
	}
	
	/**
	 * 打开语音采集
	 * @param voiceIntercomListner 语音数据采集到的回调
	 * @param eSamplePerSec 采样率
	 * @param eBitPerSample 采样深度
	 * @param eChannel 声道数
	 * @param eAudioEncType 音频编码类型
	 * @return 成功返回true，失败返回false
	 */
	public static final boolean openAudioRecord(IVoiceIntercomListener voiceIntercomListner,
												SAMPLE_PERSEC eSamplePerSec, BIT_PERSAMPLE eBitPerSample, 
												CHANNEL eChannel, AUDIO_ENC_TYPE eAudioEncType)
	{
		if (null == voiceIntercomListner)
		{
			AVErrorCenter.put(AVPLAY_ILLEGAL_PARAM);
			return false;
		}

		return VoiceIntercom.instance().start(voiceIntercomListner, eSamplePerSec, eBitPerSample, eChannel, eAudioEncType);
	}
	
	/**
	 * 停止语音采集
	 * @return 成功返回true，失败返回false
	 */
	public static final boolean closeAudioRecord()
	{
		return VoiceIntercom.instance().stop();
	}
	
	public static final void EnableImageScale(IPlayHandle hPlayHandle, boolean isScale)
	{
		if (null != hPlayHandle)
		{
			AVPlayer avPlayer = (AVPlayer)hPlayHandle;
			avPlayer.EnableImageScale(isScale);
		}
	}
	
	private static final AVPlayer getAVPlayer(AV_FILE_HEADER fileHead, int nType, 
											  IInputDataListener inputDataListener)
	{
		AVPlayer avPlayer = null;
		
		switch (nType)
		{
		case AV_STREAM_MODE_REAL_TIME:
			avPlayer = new AVRealTimePlayer(fileHead);
			break;
			
		case AV_STREAM_MODE_PLAY_BACK:
			avPlayer = new AVPlayBackPlayer(fileHead, inputDataListener);
			break;
			
		default:
			avPlayer = null;
			break;
		}
		
		return avPlayer;
	}
	
	public static void scale(IPlayHandle hPlayHandle, float scale)
	{
		if (null != hPlayHandle)
		{
			AVPlayer avPlayer = (AVPlayer)hPlayHandle;
			avPlayer.scale(scale);
		}
	}
	
	public static void translate(IPlayHandle hPlayHandle, float x, float y)
	{
		if (null != hPlayHandle)
		{
			AVPlayer avPlayer = (AVPlayer)hPlayHandle;
			avPlayer.translate(x, y);
		}
	}

	public static void setIdentity(IPlayHandle hPlayHandle)
	{
		if (null != hPlayHandle)
		{
			AVPlayer avPlayer = (AVPlayer)hPlayHandle;
			avPlayer.setIdentity();
		}
	}
	
	public static void startEPTZ(IPlayHandle hPlayHandle)
	{
		if (null != hPlayHandle)
		{
			AVPlayer avPlayer = (AVPlayer)hPlayHandle;
			avPlayer.startEPTZ();
		}
	}
	public static void stopEPTZ(IPlayHandle hPlayHandle)
	{
		if (null != hPlayHandle)
		{
			AVPlayer avPlayer = (AVPlayer)hPlayHandle;
			avPlayer.stopEPTZ();
		}
	}
	public static float getScale(IPlayHandle hPlayHandle) 
	{
		if (null != hPlayHandle)
		{
			AVPlayer avPlayer = (AVPlayer)hPlayHandle;
			return avPlayer.getScale();
		}
		return -1;
	}
	public static float getTranslateX(IPlayHandle hPlayHandle) 
	{
		if (null != hPlayHandle)
		{
			AVPlayer avPlayer = (AVPlayer)hPlayHandle;
			return avPlayer.getTranslateX();
		}
		return -1;
	}
	public static float getTranslateY(IPlayHandle hPlayHandle) 
	{
		if (null != hPlayHandle)
		{
			AVPlayer avPlayer = (AVPlayer)hPlayHandle;
			return avPlayer.getTranslateY();
		}
		return -1;
	}
}


























