﻿/*
 * Copyright (C) 2011 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.mm.android.avplaysdk.render;

import javax.microedition.khronos.egl.EGL10;
import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.egl.EGLContext;
import javax.microedition.khronos.egl.EGLDisplay;

import com.mm.android.avplaysdk.AVErrorCenter;
import com.mm.android.avplaysdk.AVPlaySDK;
import com.mm.android.avplaysdk.AVPlayer;
import com.mm.android.avplaysdk.IPlayView;
import com.mm.android.avplaysdk.IViewListener;

import android.content.Context;
import android.graphics.PixelFormat;
import android.opengl.GLSurfaceView;
import android.util.AttributeSet;
import android.util.Log;

public class BasicGLSurfaceView extends GLSurfaceView implements IPlayView
{
	private NativeGLRenderer m_glRender   = null;
	private boolean			 m_bRender	  = true;

	public BasicGLSurfaceView(Context context, AttributeSet attrs)
	{
		super(context, attrs);
	}

	public BasicGLSurfaceView(Context context)
	{
		super(context);
	}

	public Renderer getRenderer()
	{
		return m_glRender;
	}

	@Override
	public boolean init(IViewListener viewListener)
	{
		if (null == viewListener)
		{
			AVErrorCenter.put(AVPlaySDK.AVPLAY_ILLEGAL_PARAM);
			return false;
		}
		
        this.getHolder().setFormat(PixelFormat.RGBA_8888);
        setEGLContextFactory(new ContextFactory());
        setEGLConfigChooser( new ConfigChooser(8, 8, 8, 8, 0, 0));
		
		m_glRender = new NativeGLRenderer(this);
		this.setRenderer(m_glRender);
		this.setRenderMode(GLSurfaceView.RENDERMODE_WHEN_DIRTY);
		m_glRender.setViewListener(viewListener);
		
		return true;
	}

	@Override
	public void render()
	{
//		synchronized (this)
//		{
			if (m_bRender)
			{
				this.requestRender();
			}
			else
			{
				m_glRender.onlyDecode();
			}
//		}
	}

	@Override
	public void setPlayer(AVPlayer avPlayer)
	{
		m_glRender.setPlayer(avPlayer);
	}

	@Override
	public boolean stop()
	{
//		synchronized (this)
//		{
			m_glRender.stopRender();
//		}
		
		return true;
	}

	@Override
	public void enableRender(boolean bRender)
	{
		m_bRender = bRender;
		m_glRender.enableRender(bRender);
	}

	@Override
	public boolean start()
	{
		m_glRender.startRender();
		
		return true;
	}

	@Override
	public void uninit()
	{
		m_glRender.clear();
		m_glRender.deleteGLESHelp();
	}

	@Override
	public void setFrameMode(boolean bFrame) {
	}

	@Override
	public boolean getEnableRender() {
		
		return m_bRender;
	}
	
	
    private static class ContextFactory implements GLSurfaceView.EGLContextFactory {
        private static int EGL_CONTEXT_CLIENT_VERSION = 0x3098;
        public EGLContext createContext(EGL10 egl, EGLDisplay display, EGLConfig eglConfig) {
            int[] attrib_list = {EGL_CONTEXT_CLIENT_VERSION, 2, EGL10.EGL_NONE };
            EGLContext context = egl.eglCreateContext(display, eglConfig, EGL10.EGL_NO_CONTEXT, attrib_list);
            return context;
        }

        public void destroyContext(EGL10 egl, EGLDisplay display, EGLContext context) {
            egl.eglDestroyContext(display, context);
        }
    }


    private static class ConfigChooser implements GLSurfaceView.EGLConfigChooser {

        public ConfigChooser(int r, int g, int b, int a, int depth, int stencil) {
            mRedSize = r;
            mGreenSize = g;
            mBlueSize = b;
            mAlphaSize = a;
            mDepthSize = depth;
            mStencilSize = stencil;
        }

        private static int EGL_OPENGL_ES2_BIT = 4;
        private static int[] s_configAttribs2 =
        {
            EGL10.EGL_RED_SIZE, 8,
            EGL10.EGL_GREEN_SIZE, 8,
            EGL10.EGL_BLUE_SIZE, 8,
            EGL10.EGL_RENDERABLE_TYPE, EGL_OPENGL_ES2_BIT,
            EGL10.EGL_NONE
        };

        public EGLConfig chooseConfig(EGL10 egl, EGLDisplay display) {

            int[] num_config = new int[1];
            egl.eglChooseConfig(display, s_configAttribs2, null, 0, num_config);

            int numConfigs = num_config[0];

            if (numConfigs <= 0) {
                throw new IllegalArgumentException("No configs match configSpec");
            }

            EGLConfig[] configs = new EGLConfig[numConfigs];
            egl.eglChooseConfig(display, s_configAttribs2, configs, numConfigs, num_config);
            return chooseConfig(egl, display, configs);
        }

        public EGLConfig chooseConfig(EGL10 egl, EGLDisplay display,
                EGLConfig[] configs) {
            for(EGLConfig config : configs) {
                int d = findConfigAttrib(egl, display, config,
                        EGL10.EGL_DEPTH_SIZE, 0);
                int s = findConfigAttrib(egl, display, config,
                        EGL10.EGL_STENCIL_SIZE, 0);

                if (d < mDepthSize || s < mStencilSize)
                    continue;

                int r = findConfigAttrib(egl, display, config,
                        EGL10.EGL_RED_SIZE, 0);
                int g = findConfigAttrib(egl, display, config,
                            EGL10.EGL_GREEN_SIZE, 0);
                int b = findConfigAttrib(egl, display, config,
                            EGL10.EGL_BLUE_SIZE, 0);
                int a = findConfigAttrib(egl, display, config,
                        EGL10.EGL_ALPHA_SIZE, 0);

                if (r == mRedSize && g == mGreenSize && b == mBlueSize && a == mAlphaSize)
                    return config;
            }
            return null;
        }

        private int findConfigAttrib(EGL10 egl, EGLDisplay display,
                EGLConfig config, int attribute, int defaultValue) {

            if (egl.eglGetConfigAttrib(display, config, attribute, mValue)) {
                return mValue[0];
            }
            return defaultValue;
        }

        protected int mRedSize;
        protected int mGreenSize;
        protected int mBlueSize;
        protected int mAlphaSize;
        protected int mDepthSize;
        protected int mStencilSize;
        private int[] mValue = new int[1];
    }
    
	public void scale(float scale)
	{
		if(m_glRender != null)
		{
			m_glRender.scale(scale);
		}
	}
    
	public void preScale(float scale)
	{
		if(m_glRender != null)
		{
			m_glRender.preScale(scale);
		}
	}
	
	public void translate( float x, float y)
	{
		if(m_glRender != null)
		{
			m_glRender.translate(x, y);
		}
	}

	public void setIdentity()
	{
		if(m_glRender != null)
		{
			m_glRender.setIdentity();
		}
	}
	
	public void startEPTZ()
	{
		if(m_glRender != null)
		{
			m_glRender.startEPTZ();
		}
	}
	
	public void stopEPTZ()
	{
		if(m_glRender != null)
		{
			m_glRender.stopEPTZ();
		}
	}
	
	public float getScale() 
	{
		if(m_glRender != null)
		{
			return m_glRender.getScale();
		}
		return -1;
	}
	
	public float getTranslateX() 
	{
		if (m_glRender != null)
		{
			return m_glRender.getTranslateX();
		}
		return -1;
	}
	
	public float getTranslateY() 
	{
		if (m_glRender != null)
		{
			return m_glRender.getTranslateY();
		}
		return -1;
	}
}
